#include <acpi/acpi_drivers.h>
#include <acpi/processor.h>
#include <linux/dmi.h>

#define PREFIX			"ACPI: "
#define _COMPONENT		ACPI_PROCESSOR_COMPONENT
ACPI_MODULE_NAME("processor_pdc");

/*
 * _PDC is required for a BIOS-OS handshake for most of the newer
 * ACPI processor features.
 */
static int acpi_processor_eval_pdc(struct acpi_processor *pr)
{
	struct acpi_object_list *pdc_in = pr->pdc;
	acpi_status status = AE_OK;

	if (!pdc_in)
		return status;

	status = acpi_evaluate_object(pr->handle, "_PDC", pdc_in, NULL);

	if (ACPI_FAILURE(status))
		ACPI_DEBUG_PRINT((ACPI_DB_INFO,
		    "Could not evaluate _PDC, using legacy perf. control.\n"));

	return status;
}

void acpi_processor_set_pdc(struct acpi_processor *pr)
{
	arch_acpi_processor_init_pdc(pr);
	acpi_processor_eval_pdc(pr);
}
EXPORT_SYMBOL_GPL(acpi_processor_set_pdc);

static int early_pdc_optin;
static int set_early_pdc_optin(const struct dmi_system_id *id)
{
       early_pdc_optin = 1;
       return 0;
}

static struct dmi_system_id __cpuinitdata early_pdc_optin_table[] = {
       {
       set_early_pdc_optin, "HP Envy", {
       DMI_MATCH(DMI_BIOS_VENDOR, "Hewlett-Packard"),
       DMI_MATCH(DMI_PRODUCT_NAME, "HP Envy") }, NULL},
       {
       set_early_pdc_optin, "HP Pavilion dv6", {
       DMI_MATCH(DMI_BIOS_VENDOR, "Hewlett-Packard"),
       DMI_MATCH(DMI_PRODUCT_NAME, "HP Pavilion dv6") }, NULL},
       {
       set_early_pdc_optin, "HP Pavilion dv7", {
       DMI_MATCH(DMI_BIOS_VENDOR, "Hewlett-Packard"),
       DMI_MATCH(DMI_PRODUCT_NAME, "HP Pavilion dv7") }, NULL},
       {},
};

static acpi_status
early_init_pdc(acpi_handle handle, u32 lvl, void *context, void **rv)
{
	struct acpi_processor pr;

	pr.handle = handle;

	/* x86 implementation looks at pr.id to determine some
	 * CPU capabilites. We can just hard code to 0 since we're
	 * assuming the CPUs in the system are homogenous and all
	 * have the same capabilities.
	 */
	pr.id = 0;

	acpi_processor_set_pdc(&pr);

	return AE_OK;
}

void acpi_early_processor_set_pdc(void)
{
	acpi_walk_namespace(ACPI_TYPE_PROCESSOR, ACPI_ROOT_OBJECT,
			    ACPI_UINT32_MAX,
			    early_init_pdc, NULL, NULL);
}
